/*
    cmd.c
    Copyright (C) 2007 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      03.01.2011
   ----------------------------------------------------------------------------
   
   Der Sourcecode zum RF-Betrieb des SCART-Adapters stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden nicht
   bentigte Funktionen aus Grnden der Speichergre und Laufzeitoptimierung
   komplett entfernt.
   
   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts. Hierbei
   werden Daten 1:1 durchgeroutet. Die Funktionen beschrnken sich auf den
   Paket-Typ RF_PACKET_BINARY.
   ============================================================================
   Historie:
   03.01.2010  Version 1  Erste vollstndige Version nach Optimierung
   
   ============================================================================
*/

/* =================================== Includes ============================ */

#include "serial.h"
#include "cc1100.h"
#include <P89LPC932.h>
#include "cmd.h"
#include "rf.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#define RFSTARLIGHT_DEBUG   0   // Debug something on RS232


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: cmdStarLightSendRF
   ----------------------------------------------------------------------------
*/
void cmdStarLightSendRF (  )
{

   /* ----- lokale Variablen ----- */
   struct cc1100frame_  header;
   unsigned char        len;
   unsigned char        data_counter;
   unsigned char        temp_data;


   /* ----- Programmcode ----- */
   #if RFSTARLIGHT_DEBUG
      send_string( "\nEnter cmdStarLightSendRF\n");
   #endif

   while(!inbuflen);
   len = get_inbuf();

   #if RFSTARLIGHT_DEBUG
      send_string( "Laenge: ");
      send_hexC( len );
      send_string( "\n");
   #endif

   sendWOR(0);
   EKBI = 0;
   cc1100_strobe(SIDLE);
   while (cc1100_read1(0x35 | READ | BURST) != 01);
   header.len = len + headerLen - 1;
   header.destAddr = 0x00;
   header.srcAddr = cc1100Addr;
   header.packetType = RF_PACKET_BINARY;
   cc1100_write (TX_fifo, (unsigned char *)&header, headerLen);
   cc1100_single(TX_fifo | BURST, 0);

   #if RFSTARLIGHT_DEBUG
      send_string( "Senden:\n" );
   #endif

   for ( data_counter = 0; data_counter < len; data_counter++ )
   {
      while(!inbuflen);
      temp_data = get_inbuf();

      #if RFSTARLIGHT_DEBUG
         send_hexC( temp_data );
         send_string( "\n");
      #endif

      cc1100_single ( temp_data, 0);
   }

   #if RFSTARLIGHT_DEBUG
      send_string( "**\n" );
   #endif

   CS = 1;
   cc1100_strobe(STX);
   EKBI = 1;

}
/* ----- Ende cmdStarLightSendRF ------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: transmit
   ----------------------------------------------------------------------------
*/
void transmit(unsigned char len)
{

   /* ----- lokale Variablen ----- */
   struct cc1100frame_ header;


   /* ----- Programmcode ----- */
   sendWOR(0);
   EKBI = 0;
   cc1100_strobe(SIDLE);
   while (cc1100_read1(0x35 | READ | BURST) != 01);
   header.len = len + headerLen - 1;
   header.destAddr = 0;
   header.srcAddr = cc1100Addr;
   header.packetType = RF_PACKET_TEST;
   cc1100_write(TX_fifo,(unsigned char *)&header,headerLen);
   cc1100_single(TX_fifo | BURST,0);
   while(len--)
   {
      cc1100_single(get_inbuf(),0);
   }
   CS = 1;
   cc1100_strobe(STX);
   EKBI = 1;

   send_string("TX Done");

}
/* ----- Ende transmit ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: transmitRedir
   ----------------------------------------------------------------------------
*/
void transmitRedir(unsigned char len, unsigned char dest)
{

   /* ----- lokale Variablen ----- */
   struct cc1100frame_ header;


   /* ----- Programmcode ----- */
   header.len = len + headerLen - 1;
   header.destAddr = dest;
   header.srcAddr = cc1100Addr;
   header.packetType = RF_PACKET_REDIR_DAT;
   EKBI = 0;
   cc1100_write(TX_fifo,(unsigned char *)&header,headerLen);
   cc1100_single(TX_fifo | BURST,0);
   while(len--)
   {
      cc1100_single(get_inbuf(),0);
   }
   CS = 1;
   cc1100_strobe(STX);
   
   EKBI = 1;

}
/* ----- Ende transmitRedir ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: transmitB
   ----------------------------------------------------------------------------
*/
void transmitB(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char l;
   unsigned char x;
   unsigned char i;


   /* ----- Programmcode ----- */
   while(!inbuflen);

   l=get_inbuf();
   if (l < 62)
   {
      
      cc1100_single(TX_fifo ,0);
      cc1100_single(l ,0);
      for(i=0; i<l; i++)
      {
         while(!inbuflen);
         x=get_inbuf();
         cc1100_single(x ,0 );
      }
      CS = 1;
      cc1100_strobe(STX);
      waitTX();   
   }
   else
   {
      send_byte(0x00);
   }

}
/* ----- Ende transmitB ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: print_cc1100confi
   ----------------------------------------------------------------------------
*/
void print_cc1100confi(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   cc1100_single(0x00 | BURST | READ,0);
   for (x=0 ;x < 0x2F; x++)
   {  
      send_string("Reg 0x");
      send_hexC(x);
      send_string(" = 0x");
      send_hexC(cc1100_single(0x00,0));
      if ( (x & 3) == 3)
         send_string(crlf);
      else 
         send_string("   ");
   }
   CS = 1;

   send_string(crlf);
   cc1100_single(PATABLE | BURST | READ,0);
   for (x=0 ;x < 0x08; x++)
   {  
      send_string("PATABLE[");
      send_hexC(x);
      send_string("] = 0x");
      send_hexC(cc1100_single(0x00,0));
      if ( (x & 3) == 3)
         send_string(crlf);
      else 
         send_string("  ");
   }
   CS = 1;

}
/* ----- Ende print_cc1100confi -------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: print_cc1100status
   ----------------------------------------------------------------------------
*/
void print_cc1100status(void)
{

   /* ----- Programmcode ----- */
   send_string("CC1100 Read Status: ");
   send_hexC(cc1100_strobe(SNOP | READ));
   send_string("  Write Status: ");
   send_hexC(cc1100_strobe(SNOP | WRITE));

}
/* ----- Ende print_cc1100status -------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: read_byteB
   ----------------------------------------------------------------------------
*/
void read_byteB(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   x=get_inbuf_wait();
   cc1100_single(x | READ,0);
   x = cc1100_single(0,1);
   send_byte(x);

}
/* ----- Ende read_byteB --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: write_byteB
   ----------------------------------------------------------------------------
*/
void write_byteB(void)
{


   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   x=get_inbuf_wait();
   cc1100_single(x | WRITE ,0);
   while (!RI);
   x = SBUF;
   RI = 0;
   cc1100_single(x,1);

}
/* ----- Ende write_byteB -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: read_burst_byteB
   ----------------------------------------------------------------------------
*/
void read_burst_byteB(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char l;
   unsigned char x;
   unsigned char i;


   /* ----- Programmcode ----- */
   l=get_inbuf_wait();
   x=get_inbuf_wait();
   
   cc1100_single(x | READ | BURST ,0);
   for(i=0; i<l; i++)
   {
      x= cc1100_single(0 ,0 );
      send_byte(x);
   }
   CS = 1;

}
/* ----- Ende read_burst_byteB --------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: write_burst_byteB
   ----------------------------------------------------------------------------
*/
void write_burst_byteB(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char l;
   unsigned char x;
   unsigned char i;


   /* ----- Programmcode ----- */
   l=get_inbuf_wait();
   x=get_inbuf_wait();
   
   cc1100_single(x | WRITE | BURST ,0);
   for(i=0; i<l; i++)
   {
      while (!RI);
         x = SBUF;
         RI = 0;
         cc1100_single(x ,0 );
   }
   CS = 1;

}
/* ----- Ende write_burst_byteB -------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cmd_strobeB
   ----------------------------------------------------------------------------
*/
void cmd_strobeB(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   x=get_inbuf_wait();
   
   send_byte(cc1100_strobe(x));

}
/* ----- Ende cmd_strobeB -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: send_WORb
   ----------------------------------------------------------------------------
*/
void send_WORb(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   x=get_inbuf_wait();
   sendWOR(x);

}
/* ----- Ende send_WORb ---------------------------------------------------- */


/* =================================== Ende ================================ */

