/*
    rf.c
    Copyright (C) 2009 Telekatz <telekatz@gmx.de> 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      03.01.2011
   ----------------------------------------------------------------------------
   
   Der Sourcecode zum RF-Betrieb des SCART-Adapters stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden nicht
   bentigte Funktionen aus Grnden der Speichergre und Laufzeitoptimierung
   komplett entfernt.
   
   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts. Hierbei
   werden Daten 1:1 durchgeroutet. Die Funktionen beschrnken sich auf den
   Paket-Typ RF_PACKET_BINARY.
   ============================================================================
   Historie:
   03.01.2010  Version 1  Erste vollstndige Version nach Optimierung
   
   ============================================================================
*/

/* =================================== Includes ============================ */

#include <P89LPC932.h>
#include "cc1100.h"
#include "cmd.h"
#include "rf.h"
#include "serial.h"


/* ================== Datentypen und Variablendefinitionen ================= */

bit WORsend;
unsigned char cc1100Addr;


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: RF_init
   ----------------------------------------------------------------------------
*/
void RF_init (void)
{

   /* ----- Programmcode ----- */
   cc1100_init();
   WORsend=0;
   cc1100Addr = conf[0x09];
   
}
/* ----- Ende RF_init ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: RF_startIRQ
   ----------------------------------------------------------------------------
*/
void RF_startIRQ (void)
{

   /* ----- Programmcode ----- */
   KBPATN = 0x40;
   KBMASK = 0x40;
   KBCON = 0x02;
   EKBI = 1;
   
}
/* ----- Ende RF_startIRQ -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: RF_isr
   ----------------------------------------------------------------------------
*/
void RF_isr (void) __interrupt (7)
{
   /* ----- lokale Variablen ----- */
   struct cc1100frameheader_  RXframe;
   unsigned char              i;


   /* ----- Programmcode ----- */
   if(KBPATN & 0x40)
   {
      KBPATN = 0x00;      //IRQ on low level
      KBCON = 0x02;
      return;
   }
   
   RXframe.len = cc1100_read1(RXBYTES);
   if ( RXframe.len )
   {
      if(RXframe.len > 3)
      {
         cc1100_read(RX_fifo, &RXframe.len,4);
      }
      else
      {
         cc1100_read(RX_fifo, &RXframe.len,RXframe.len);
      }

      if ( RXframe.len > 3 )
      {
            bit ACK = 0;

            // RF_PACKET_TEST         1
            // RF_PACKET_PING         2
            // RF_PACKET_TIME         3
            // RF_PACKET_RF_ENC       4
            // RF_PACKET_REDIR_DAT    5
            // RF_PACKET_REDIR_CMD    6
            // RF_PACKET_XML          7
            // RF_PACKET_BINARY       8
            
            // struct cc1100frameheader_ {
            // unsigned char len;
            // unsigned char destAddr;
            // unsigned char srcAddr;
            // unsigned char packetType; }; 

            switch ( RXframe.packetType )
            {

               //RF_PACKET_BINARY      0x08
               case RF_PACKET_BINARY:
                  // Binre Datenbertragung
                  // 0x02 <n> <data_1> ... <data_n>          n bytes data received ber Funktion RF_PACKET_BINARY

                  send_byte( 0x02 );
                  send_bytes(&RXframe.len, 4);
                  cc1100_single(RX_fifo | BURST | READ,0);
                  for(i=3;i<RXframe.len;i++)
                  {
                     send_byte(cc1100_single(0,0));
                  }
                  CS = 1;
                  break;

               default:
                  send_byte(0x03);
                  send_bytes(&RXframe.len, 4);
                  cc1100_single(RX_fifo | BURST | READ,0);
                  for(i=3;i<RXframe.len;i++)
                  {
                     send_byte(cc1100_single(0,0));
                  }
                  CS = 1;
                  break;
                  
            } // switch
      } // Frame > 3
      
      cc1100_strobe(SIDLE);
      while (cc1100_read1(MARCSTATE) != MARCSTATE_IDLE);
      cc1100_strobe(SFRX);
      cc1100_strobe( SRX );
   }
   else 
   {
      if(cc1100_read1(MARCSTATE) != MARCSTATE_RX)
      {
         cc1100_strobe(SIDLE);
         while (cc1100_read1(MARCSTATE) != MARCSTATE_IDLE);
         cc1100_strobe(SFRX);
         cc1100_strobe(SFTX);
         cc1100_strobe(SRX);
      }
   }
   
   KBPATN = 0x40;      //IRQ on high level
   KBCON = 0x02;
}
/* ----- Ende RF_isr ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: waitTX
   ----------------------------------------------------------------------------
*/
void waitTX(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char status;
   unsigned char x;


   /* ----- Programmcode ----- */
   x=1;
   while (x) {
      status = cc1100_strobe(SNOP);
      switch ( 0xf0 & status)
      {
         case 0x70:
               cc1100_strobe(SFTX);
            break;
         case  0x10:
            if (WORsend)
               cc1100_strobe(SIDLE);
            x=0;
            break;
         case  0x00:
            if (!(WORsend))
               cc1100_strobe(SRX);
            x=0;
            break;
      }   
   }
}
/* ----- Ende waitTX ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: sendWOR
   ----------------------------------------------------------------------------
*/
void sendWOR(unsigned char addr)
{

   /* ----- lokale Variablen ----- */
   unsigned char  b[2];
   unsigned int   i;


   /* ----- Programmcode ----- */
   b[0]=0x01;
   b[1]=addr;
   
   EKBI = 0;
   WORsend=1;
   cc1100_write1(0x18,conf[0x18] & 0xCF);
   
   cc1100_strobe(SIDLE);
   cc1100_strobe(SCAL);
   waitTX();
   
   i=0;
   while (i++ < 250)
   {
      cc1100_write(TX_fifo | BURST,b,2);
      cc1100_strobe(STX);
      waitTX();
   }
   
   cc1100_write1(0x18,conf[0x18]);
   WORsend=0;
   waitTX();
   KBCON = 0x02;
   EKBI = 1;
   
}
/* ----- Ende sendWOR ------------------------------------------------------ */


/* =================================== Ende ================================ */

