/*
    serial.c
    Copyright (C) 2007 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      03.01.2011
   ----------------------------------------------------------------------------
   
   Der Sourcecode zum RF-Betrieb des SCART-Adapters stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden nicht
   bentigte Funktionen aus Grnden der Speichergre und Laufzeitoptimierung
   komplett entfernt.
   
   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts. Hierbei
   werden Daten 1:1 durchgeroutet. Die Funktionen beschrnken sich auf den
   Paket-Typ RF_PACKET_BINARY.
   ============================================================================
   Historie:
   03.01.2010  Version 1  Erste vollstndige Version nach Optimierung
   
   ============================================================================
*/

/* =================================== Includes ============================ */

#include "serial.h"
#include "cmd.h"
#include <P89LPC932.h>


/* ================== Datentypen und Variablendefinitionen ================= */

unsigned char           inbufindex = 0;
volatile unsigned char  inbuflen = 0;
idata unsigned char     inbuf[64];

unsigned char           outbufindex;
volatile unsigned char  outbuflen;
idata unsigned char     outbuf[64];

const unsigned char      baudtable[10][2] = { { 45,  27},      //   1200
                                              { 22, 133},      //   2400
                                              { 11,  59},      //   4800
                                              {  5, 149},      //   9600
                                              {  3, 180},      //  14400
                                              {  2, 195},      //  19200
                                              {  1, 210},      //  28800
                                              {  1,  89},      //  38400
                                              {  0, 225},      //  57600
                                              {  0, 104}};     // 115200


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: setbaud
   ----------------------------------------------------------------------------
*/
void setbaud(unsigned char baud)
{

   /* ----- Programmcode ----- */
   BRGR0 = baudtable[baud][1];
   BRGR1 = baudtable[baud][0];

}
/* ----- Ende setbaud ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: initSerial
   ----------------------------------------------------------------------------
*/
void initSerial(void)
{

   /* ----- Programmcode ----- */
   SCON   = (1<<4) | (1<<6) | (1<<1);   // Mode1; Enable serial reception
   SSTAT |= (1<<7) | (1<<5);                     
   setbaud(baud_default);
   BRGCON = 0x03;
   RI = 0;
   ESR = 1;                        //Enable serial receive interrupt
   TI = 1;   
   PSR = 1;                        //Serial receive interrupt priority level 1
   PST = 1;                        //Serial transmit interrupt priority level 1

}
/* ----- Ende initSerial --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: send_string
   ----------------------------------------------------------------------------
*/
void send_string(const unsigned char* string)
{

   /* ----- lokale Variablen ----- */
   unsigned char i=0;


   /* ----- Programmcode ----- */
   while (string[i] != 0)
   {
      send_byte(string[i++]);
   } 

}
/* ----- Ende send_string -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: send_bytes
   ----------------------------------------------------------------------------
*/
void send_bytes(unsigned char* h, unsigned char l)
{

   /* ----- lokale Variablen ----- */
   unsigned char i=0;


   /* ----- Programmcode ----- */
   while (i<l)
   {
      send_byte(h[i++]);
   } 

}
/* ----- Ende send_bytes --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: send_byte
   ----------------------------------------------------------------------------
*/
void send_byte(unsigned char h)
{

   /* ----- Programmcode ----- */
   while(!put_outbuf(h))
   {
      EST = 1;
   }
   EST = 1;

}
/* ----- Ende send_byte ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: send_hexC
   ----------------------------------------------------------------------------
*/
void send_hexC(unsigned char c)
{

   /* ----- lokale Variablen ----- */
   unsigned char cn;


   /* ----- Programmcode ----- */
   cn = (c>>4) & 0x0f;
   cn += 0x30;
   if (cn > 0x39)
      cn += 0x07;
   send_byte(cn);
   
   cn = c & 0x0f;
   cn += 0x30;
   if (cn > 0x39)
      cn += 0x07;
   send_byte(cn);

} 
/* ----- Ende send_hexC ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_in_isr
   ----------------------------------------------------------------------------
*/
void serial_in_isr (void) __interrupt (4)
{

   /* ----- Programmcode ----- */
   if (inbuflen < 64)
   {
      put_inbuf(SBUF);
   }
   RI = 0;
   TF0 = 0;
   TH0 = 0;

}
/* ----- Ende serial_in_isr ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_out_isr
   ----------------------------------------------------------------------------
*/
void serial_out_isr (void) __interrupt (13)
{

   /* ----- Programmcode ----- */
   if (outbuflen)
   {
      SBUF = get_outbuf();
      TI=0;
   }
   else
   {
      EST = 0;
   }   

}
/* ----- Ende serial_out_isr ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: get_inbuf
   ----------------------------------------------------------------------------
*/
unsigned char get_inbuf(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char x = 0;


   /* ----- Programmcode ----- */
   EA = 0;
   if(inbuflen)
   {
      inbuflen--;
      x = inbuf[(inbufindex++) & 0x3F];
   }
   EA = 1;
   return(x);

}
/* ----- Ende get_inbuf ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: get_outbuf
   ----------------------------------------------------------------------------
*/
unsigned char get_outbuf(void)
{
   /* ----- lokale Variablen ----- */
   unsigned char x = 0;


   /* ----- Programmcode ----- */
   EA = 0;
   if(outbuflen)
   {
      outbuflen--;
      x = outbuf[(outbufindex++) & 0x3F];
   }
   EA = 1;
   return(x);

}
/* ----- Ende get_outbuf --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: get_inbuf_wait
   ----------------------------------------------------------------------------
*/
unsigned char get_inbuf_wait(void)
{

   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   while(!inbuflen);
   EA = 0;
   inbuflen--;
   x = inbuf[(inbufindex++) & 0x3F];
   EA = 1;
   return(x);

}
/* ----- Ende get_inbuf_wait ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: get_inbuf_index
   ----------------------------------------------------------------------------
*/
unsigned char get_inbuf_index(unsigned char index)
{
   
   /* ----- Programmcode ----- */
   return(inbuf[(inbufindex + index) & 0x3F]);

}
/* ----- Ende get_inbuf_index ---------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: put_inbuf
   ----------------------------------------------------------------------------
*/
void put_inbuf(unsigned char dat)
{

   /* ----- Programmcode ----- */
   if (inbuflen < 64)
      inbuf[(inbufindex + inbuflen++) & 0x3F] = dat;

}
/* ----- Ende put_inbuf ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: put_outbuf
   ----------------------------------------------------------------------------
*/
unsigned char put_outbuf(unsigned char dat)
{

   /* ----- Programmcode ----- */
   EA = 0;
   if (outbuflen < 64)
   {
      outbuf[(outbufindex + outbuflen++) & 0x3F] = dat;
      EA = 1;
      return 1;
   }
   EA = 1;
   return 0;

}
/* ----- Ende put_outbuf --------------------------------------------------- */


/* =================================== Ende ================================ */

